@@ -1,5 +1,6 @@
+import json
 from six import iteritems
-from traitlets import HasTraits, Unicode, List, Bool, Int, Dict, Any, validate
+from traitlets import HasTraits, Unicode, List, Bool, Int, Dict, Any, Bytes, validate
 from .data import type_detect
 from .validate import validate_view, validate_columns, validate_rowpivots, validate_columnpivots, validate_aggregates, validate_sort, validate_computedcolumns
 from .schema import validate_schema
@@ -9,7 +10,7 @@ class PerspectiveBaseMixin(HasTraits):
     '''Perspective Base Mixin'''
     # Data (private)
     _data = List(default_value=[]).tag(sync=True)
-    _bin_data = Any().tag(sync=True)
+    _bin_data = Bytes().tag(sync=True)  # binary data
     _dat_orig = Any()
 
     # Data source
@@ -37,9 +38,9 @@ class PerspectiveBaseMixin(HasTraits):
     dark = Bool(False).tag(sync=True)
 
     def load(self, value):
-        data_object = type_detect(value)
+        data_object = type_detect(value, schema=self.schema)
         self.datasrc = data_object.type
-        if data_object.type in ('pyarrow'):
+        if data_object.type in ('arrow'):
             self._data = []
             self._bin_data = data_object.data
             return
@@ -72,7 +73,7 @@ class PerspectiveBaseMixin(HasTraits):
 
                 self.columns = columns + computedcolumns
 
-            elif 'columns' in data_object.kwargs:
+            elif not self.columns and 'columns' in data_object.kwargs:
                 columns = list(map(lambda x: str(x), data_object.kwargs.pop('columns')))
                 self.columns = columns + computedcolumns
 
@@ -80,7 +81,8 @@ class PerspectiveBaseMixin(HasTraits):
             self.schema = {}
 
         for k, v in iteritems(data_object.kwargs):
-            setattr(self, k, v)
+            if not getattr(self, k):
+                setattr(self, k, v)
 
         # set data last
         self._data = data_object.data
@@ -112,6 +114,28 @@ class PerspectiveBaseMixin(HasTraits):
     @validate('computedcolumns')
     def _validate_computedcolumns(self, proposal): return validate_computedcolumns(proposal.value)
 
+    def _as_json(self, data_only=False, allow_nan=False):
+        ret = {}
+        ret['datasrc'] = getattr(self, 'datasrc')
+        ret['data'] = getattr(self, '_data')
+        if data_only:
+            return json.dumps(ret, allow_nan=allow_nan)
+
+        ret['schema'] = getattr(self, 'schema')
+        ret['view'] = getattr(self, 'view')
+        ret['columns'] = getattr(self, 'columns')
+        ret['rowpivots'] = getattr(self, 'rowpivots')
+        ret['columnpivots'] = getattr(self, 'columnpivots')
+        ret['aggregates'] = getattr(self, 'aggregates')
+        ret['sort'] = getattr(self, 'sort')
+        ret['index'] = getattr(self, 'index')
+        ret['limit'] = getattr(self, 'limit')
+        ret['computedcolumns'] = getattr(self, 'computedcolumns')
+        ret['settings'] = getattr(self, 'settings')
+        ret['embed'] = getattr(self, 'embed')
+        ret['dark'] = getattr(self, 'dark')
+        return json.dumps(ret, allow_nan=allow_nan)
+
     def setup(self,
               data,
               view='hypergrid',
