@@ -5,9 +5,10 @@ from .pa import _is_pyarrow
 EXPORTERS = [_is_dict, _is_list, _is_pandas, _is_pyarrow]
 
 
-def type_detect(data):
+def type_detect(data, schema=None):
+    schema = None or {}
     for foo in EXPORTERS:
-        data_object = foo(data)
+        data_object = foo(data, schema)
         if data_object.type:
             return data_object
     # throw error?
